//-----------------------------------------------------------------------------------
//                     MATRIX CLASS --- by GERNOT MUELLER, 2004
//-----------------------------------------------------------------------------------
// most important available operators: ! (inversion), *, +, -
// most important available functions:
// FUNCTION NAME              VALUE
// transp                     transposed matrix
// Cholesky                   Cholesky matrix
// invCholesky                inverse of Cholesky matrix (fast implementation!)
// invertPosDef               inverse of pos. definit matrix (fast implementation!)
// tmat2                      submatrix
// rang                       rank of matrix
// det                        determinant of matrix
// posdef                     1, if matrix is pos. definit, 0 else
//-----------------------------------------------------------------------------------

#ifndef __MATRIXGM_H
#define __MATRIXGM_H

#include <iostream.h>
#include <assert.h>
#include <math.h>

//#define SPEC huge
#define SPEC
int max(int i, int k);

//-----------------------------------------------------------------------------------
// class declaration

class Matrix
{
public:
   // constructor
   Matrix(unsigned zeilen, unsigned spalten);
      
   // copy constructor
   Matrix(Matrix &toCopy);
   
   // destructor
   ~Matrix() { delete[] M; }

   // return element, e.g. A(1,2);
   double & operator ()(unsigned zeile, unsigned spalte);
  
   // assignment operator
   Matrix & operator =(const Matrix& toAssign);
   
   // unary plus 
   Matrix operator +();
   
   // unary minus
   Matrix operator -();
    
   // addition
   friend Matrix operator +(const Matrix& A, const Matrix& B);

   // subtraction
   friend Matrix operator -(const Matrix& A, const Matrix& B);

   // multiplication by matrix
   friend Matrix operator *(const Matrix& A, const Matrix& B);

   // multiplication by scalar
   friend Matrix operator *(const double f, const Matrix& B);

   // transpose matrix
   Matrix transp(void);
   
   // invert matrix
   friend Matrix operator !(const Matrix& A);

   // Cholesky-decomposition
   Matrix Cholesky(void);

   // invert Cholesky-matrix
   Matrix invCholesky(void);

   // invert positive definite matrix
   Matrix invertPosDef(void);

   // change rows i and k
   Matrix change_rows(unsigned i, unsigned k);

   // change columns i and k
   Matrix change_cols(unsigned i, unsigned k);
   
   // multiply row z by double f
   Matrix mult_r(double f, unsigned z);
   
   // multiply column s by double f
   Matrix mult_c(double f, unsigned s);
   
   // add row i to row k
   Matrix add_r(unsigned i, unsigned k);
   
   // left top ixk-submatrix
   Matrix tmat(unsigned i, unsigned k);

   // submatrix (i,k) to (j,l)
   Matrix tmat2(unsigned i, unsigned k, unsigned j, unsigned l);

   // compute rank
   int rang(void);

   // compute determinant (Gauss-elimination)
   double det(void);

   // test whether matrix is positive definite (0=no, 1=yes)
   int posdef(void);
   
public:
   unsigned m,n;
   SPEC double *M;
};


#endif



